/************************************************/
/*                                              */
/*  MSXCAS                                      */
/*  MSX to IBMPC/XT/AT etc file converter       */
/*  KORVET PK8020/PK8010 version                */
/*                                              */
/*  Version 2.1                                 */
/*  Copyright (C) MarinovSoft 2009-2011         */
/*                                              */
/*  From Kuzbass (Siberia) with Love            */
/*                                              */
/*  http://marinovsoft.narod.ru                 */
/*  mailto:super386@rambler.ru                  */
/*                                              */
/************************************************/

#pragma option -a1

#define cmAbout         170
#define cmRAWProcess    171
#define cmBINProcess    172
#define cmBASProcess    173
#define cmTXTProcess    174
#define cmTX1Process    175
#define cmTX2Process    176
#define cmCASProcess    177
#define cmFileNew       178
#define cmOptions       179

#define Uses_TApplication
#define Uses_TKeys
#define Uses_MsgBox
#define Uses_TRect
#define Uses_TMenuBar
#define Uses_TSubMenu
#define Uses_TMenuItem
#define Uses_TStatusLine
#define Uses_TStatusItem
#define Uses_TStatusDef
#define Uses_TDeskTop
#define Uses_TFileDialog
#define Uses_TMsgBox

#include <tv.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include "appwin.h"
#include "service.h"
#include "part3msx.h"
#include "optdlg.h"
#include "waves.h"

//#include <process.h>

ushort execDialog(TDialog *d, void *data)
{
   TView *p = TProgram::application->validView(d);

   if(p == 0)
   {
      return(cmCancel);
   }
   else
   {
      if(data != 0)
      {
         p->setData(data);
      }
      ushort result = TProgram::deskTop->execView(p);
      if(result != cmCancel && data != 0)
      {
         p->getData(data);
      }
      TObject::destroy(p);
      return(result);
   }
}

class TKorvetApp : public TApplication
{
public:
   TKorvetApp();
   char TheFile[MAXPATH];
   TInfoWindow *InfoWindow;
   static TStatusLine *initStatusLine(TRect r);
   static TMenuBar *initMenuBar(TRect r);
   virtual void handleEvent(TEvent& event);
   virtual void DoAboutBox();
   void OpenWindow();
   void MSXFileNew();
   void Txt2Msx();
   void Cas2Msx();
   void Bin2Msx();
   void Msr2Wav();
   void RAWRecode();
   void TX1Decode();
   void TX2Decode();
   void CASDecode();
   void BASDecode();
   void setOptions();
   void Write_bit_1(FILE *_RAWFile);
   void Write_bit_0(FILE *_RAWFile);
   void Write_byte(FILE *_RAWFile, unsigned char B);
   void WriteFileName(FILE *_RAWFile, const char *MSXFileName);
protected:
   unsigned short int LoadAddr;
   unsigned short int EndAddr;
   unsigned short int StartAddr;
};
TKorvetApp::TKorvetApp() :
   TProgInit(&TKorvetApp::initStatusLine,
             &TKorvetApp::initMenuBar,
             &TKorvetApp::initDeskTop)
{
  disableCommand(cmRAWProcess);
  disableCommand(cmTX1Process);
  disableCommand(cmTX2Process);
  disableCommand(cmCASProcess);
  disableCommand(cmBASProcess);

  LoadAddr  = 0x9000;
  StartAddr = LoadAddr;
}

TStatusLine *TKorvetApp::initStatusLine(TRect r)
{
   r.a.y = r.b.y - 1;
   return(new TStatusLine(r,
         *new TStatusDef(0, 0xFFFF) +
         *new TStatusItem("~Alt-X~ Exit", kbAltX, cmQuit) +
         *new TStatusItem("~F3~ Open", kbF3, cmFileOpen) +
         *new TStatusItem("~F10~ Menu", kbF10, cmMenu)
   ));
}

TMenuBar *TKorvetApp::initMenuBar(TRect r)
{
   r.b.y = r.a.y + 1;
   return(new TMenuBar(r,
   *new TSubMenu("~F~ile", kbAltF) +
   *new TMenuItem("~N~ew", cmFileNew, kbF2, hcNoContext, "F2") +
   *new TMenuItem("~O~pen...", cmFileOpen, kbF3, hcNoContext, "F3") +
   newLine() +
   *new TMenuItem("O~p~tions", cmOptions, kbNoKey, hcNoContext, "") +
   newLine() +
   *new TMenuItem("E~x~it", cmQuit, kbAltX, hcNoContext, "Alt-X") +
   *new TSubMenu("~D~ecode", kbNoKey) +
   *new TMenuItem("~R~aw recode", cmRAWProcess, kbNoKey, hcNoContext, "") +
   *new TMenuItem("Phase ~1~ decode", cmTX1Process, kbNoKey, hcNoContext, "") +
   *new TMenuItem("Phase ~2~ decode", cmTX2Process, kbNoKey, hcNoContext, "") +
   *new TMenuItem("~C~AS/TXT/BIN decode", cmCASProcess, kbNoKey, hcNoContext, "") +
   *new TMenuItem("~B~AS decode", cmBASProcess, kbNoKey, hcNoContext, "") +

   *new TSubMenu("~W~indow", kbNoKey) +
   *new TMenuItem("~T~ile", cmResize, kbCtrlF5, hcNoContext, "") +
   *new TMenuItem("C~a~scade", cmCascade, kbNoKey, hcNoContext, "") +
   *new TMenuItem("Cl~o~se all", cmClose, kbNoKey, hcNoContext, "") +
   newLine() +
   *new TMenuItem("Size/Move", cmResize, kbNoKey, hcNoContext, "Ctrl+F5") +
   *new TMenuItem("Zoom", cmZoom, kbNoKey, hcNoContext, "F5") +
   *new TMenuItem("Next", cmNext, kbF6, hcNoContext, "F6") +
   *new TMenuItem("Previous", cmPrev, kbNoKey, hcNoContext, "Shift+F6") +
   *new TMenuItem("Close", cmClose, kbAltF3, hcNoContext, "Alt+F3") +

   *new TSubMenu("~H~elp", kbNoKey) +
   *new TMenuItem("~A~bout...", cmAbout, kbF1, hcNoContext, "F1")
   ));
}

void TKorvetApp::handleEvent(TEvent& event)
{
   if(event.what == evCommand)
   {
      switch(event.message.command)
      {
         case cmFileOpen:
            OpenWindow();
            clearEvent(event);
            break;
         case cmAbout:
            DoAboutBox();
            clearEvent(event);
            break;
         case cmRAWProcess:
            RAWRecode();
            clearEvent(event);
            break;
         case cmTX1Process:
            TX1Decode();
            clearEvent(event);
            break;
         case cmTX2Process:
            TX2Decode();
            clearEvent(event);
            break;
         case cmCASProcess:
            CASDecode();
            clearEvent(event);
            break;
         case cmBASProcess:
            BASDecode();
            clearEvent(event);
            break;
         case cmFileNew:
            MSXFileNew();
            clearEvent(event);
            break;
         case cmOptions:
            setOptions();
            clearEvent(event);
            break;
         case cmClose:
            disableCommand(cmRAWProcess);
            disableCommand(cmTX1Process);
            disableCommand(cmTX2Process);
            disableCommand(cmCASProcess);
            disableCommand(cmBASProcess);
            break;
      }
   }
   TApplication::handleEvent(event);
}

void TKorvetApp::DoAboutBox()
{
   messageBox("\3MSX File Converter v2.1\n\3Copyright (C) 2007-2011\n\3ਭ ࣨ aka MarinovSoft",
   mfInformation | mfOKButton);
}

void TKorvetApp::OpenWindow()
{
   char titul[MAXPATH];

   strcpy(TheFile, "*.RAW");
   TFileDialog *FileDialog = new TFileDialog(TheFile, "Open file", "~F~ile name", fdOpenButton, 1);
   if(execDialog(FileDialog, TheFile) != cmCancel)
   {
      strcpy(titul, "Reading file ");
      strcat(titul, TheFile);
      InfoWindow = new TInfoWindow(titul);
      deskTop->insert(InfoWindow);
      enableCommand(cmRAWProcess);
      enableCommand(cmTX1Process);
      enableCommand(cmTX2Process);
      enableCommand(cmCASProcess);
      enableCommand(cmBASProcess);
   }
}

void TKorvetApp::Write_bit_0(FILE *_RAWFile)
{
   short int bit0[36] =
   {
      -5000, -5000, -5000, -5000, -5000, -5000, -5000, -5000, -5000,
      -5000, -5000, -5000, -5000, -5000, -5000, -5000, -5000, -5000,
      5000,   5000,  5000,  5000,  5000,  5000,  5000,  5000,  5000,
      5000,   5000,  5000,  5000,  5000,  5000,  5000,  5000, 5000
   };
   fwrite(&bit0, sizeof(bit0), 1, _RAWFile);
}

void TKorvetApp::Write_bit_1(FILE *_RAWFile)
{
   short int bit1[36] =
   {
      -5000, -5000, -5000, -5000, -5000, -5000, -5000, -5000, -5000,
      5000,   5000,  5000,  5000,  5000,  5000,  5000,  5000,  5000,
      -5000, -5000, -5000, -5000, -5000, -5000, -5000, -5000, -5000,
      5000,   5000,  5000,  5000,  5000,  5000,  5000,  5000, 5000
   };
   fwrite(&bit1, sizeof(bit1), 1, _RAWFile);
}

void TKorvetApp::Write_byte(FILE *_RAWFile, unsigned char B)
{
   unsigned char arr1[8] =
   {
      0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80
   };

   Write_bit_0(_RAWFile);
   for(unsigned char i = 0; i <= 7; i++)
   {
      if((B & arr1[i]) == arr1[i])
      {
         Write_bit_1(_RAWFile);
      }
      else
      {
         Write_bit_0(_RAWFile);
      }
   }
   Write_bit_1(_RAWFile);
   Write_bit_1(_RAWFile);
}

void TKorvetApp::WriteFileName(FILE *_RAWFile, const char *MSXFileName)
{
   for(unsigned char i = 0; i <= 5; i++)
   {
      Write_byte(_RAWFile, MSXFileName[i]);
   }
}

void TKorvetApp::Txt2Msx()
{
   TProgressDialog *ProgressDialog;
   char titul[MAXPATH];
   char RAWFile[MAXPATH];
   char MSXFile[MAXPATH];
   char tmpstr[255];
   short int nul0 = 0;
   strcpy(titul, TheFile);
   strcpy(tmpstr, "|31| Reading file ");
   strcat(tmpstr, TheFile);
   InfoWindow->writeLine(tmpstr);
   ChangeFileExt(RAWFile, TheFile, ".RAW");
   strcpy(tmpstr, "|31| Convert source to MSX tape TXT file ");
   strcat(tmpstr, RAWFile);
   InfoWindow->writeLine(tmpstr);
   InfoWindow->writeLine("|31| Open this file in your Wave Editor or Player with following params:");
   InfoWindow->writeLine("|31| Sample rate: 44100 Hz");
   InfoWindow->writeLine("|31| Sample type: 16 Bit PCM");
   InfoWindow->writeLine("|31| Signed");
   InfoWindow->writeLine("|31| Byte order: Little-endian");
   InfoWindow->writeLine("|31| Chanels: Mono");
   strcpy(tmpstr, "|31| Set MSX filename to [");
   GetMSXFileName(MSXFile, RAWFile);
   strcat(tmpstr, MSXFile);
   strcat(tmpstr, "]");
   InfoWindow->writeLine(tmpstr);
   InfoWindow->writeLine("|31| In PK8020/PK8010 KORVET you can read this file:");
   strcpy(tmpstr, "|31| LOAD \"CAS:");
   strcat(tmpstr, MSXFile);
   strcat(tmpstr, "\",A");
   InfoWindow->writeLine(tmpstr);

   FILE *_TheFile = fopen(TheFile, "r+b");
   if (_TheFile==NULL)
   {
      strcpy(tmpstr,"|31| Error reading file: ");
      strcat(tmpstr,TheFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   FILE *_RAWFile = fopen(RAWFile, "w+b");
   if (_RAWFile==NULL)
   {
      strcpy(tmpstr,"|31| Can't create file: ");
      strcat(tmpstr,RAWFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   ProgressDialog = new TProgressDialog("Convert TXT file");
   deskTop->insert(ProgressDialog);

   fseek(_TheFile, 0, 2);
   long fz = ftell(_TheFile);
   fseek(_TheFile, 0, 0);

   for(unsigned short int w = 1; w <= 0x1E00; w++)
   {
       Write_bit_1(_RAWFile);
   }
   for(w = 1; w <= 10; w++)
   {
       Write_byte(_RAWFile, 0xEA);
   }
   WriteFileName(_RAWFile, MSXFile);

   unsigned char j = 0;
   unsigned char B;
   unsigned long fpos = 0;
   while(fpos < fz)
   {
      if(j == 0)
      {
         for(w = 0; w != 0xFFFF; w++)
         {
            fwrite(&nul0, sizeof(nul0), 1, _RAWFile);
         }
         for(w = 0; w != 0x780; w++)
         {
            Write_bit_1(_RAWFile);
         }
      }
      fread(&B, sizeof(B), 1, _TheFile);
      Write_byte(_RAWFile, B);
      fpos = ftell(_TheFile);
      ProgressDialog->update((100 * fpos) / fz);
      j++;
   }

   if(j != 0)
   {
      for(B = j; (B != 0x00); B++)
      {
         Write_byte(_RAWFile, 0x1A);
      }
   }
   Write_byte(_RAWFile, 0x1A);

   fclose(_TheFile);
   fclose(_RAWFile);
   destroy(ProgressDialog);
   InfoWindow->writeLine("|31| Successfully!!!");
}

void TKorvetApp::Cas2Msx()
{
   TProgressDialog *ProgressDialog;
   char titul[MAXPATH];
   char RAWFile[MAXPATH];
   char MSXFile[MAXPATH];
   char tmpstr[255];
   short int nul0 = 0;

   strcpy(titul, TheFile);
   strcpy(tmpstr, "|31| Reading file ");
   strcat(tmpstr, TheFile);
   InfoWindow->writeLine(tmpstr);
   ChangeFileExt(RAWFile, TheFile, ".RAW");
   strcpy(tmpstr, "|31| Convert source to MSX tape BASIC file ");
   strcat(tmpstr, RAWFile);
   InfoWindow->writeLine(tmpstr);
   InfoWindow->writeLine("|31| Open this file in your Wave Editor or Player with following params:");
   InfoWindow->writeLine("|31| Sample rate: 44100 Hz");
   InfoWindow->writeLine("|31| Sample type: 16 Bit PCM");
   InfoWindow->writeLine("|31| Signed");
   InfoWindow->writeLine("|31| Byte order: Little-endian");
   InfoWindow->writeLine("|31| Chanels: Mono");
   strcpy(tmpstr, "|31| Set MSX filename to [");
   GetMSXFileName(MSXFile, RAWFile);
   strcat(tmpstr, MSXFile);
   strcat(tmpstr, "]");
   InfoWindow->writeLine(tmpstr);
   InfoWindow->writeLine("|31| In PK8020/PK8010 KORVET you can read this file:");
   strcpy(tmpstr, "|31| LOAD \"CAS:");
   strcat(tmpstr, MSXFile);
   strcat(tmpstr, "\"[,R]");
   InfoWindow->writeLine(tmpstr);

   FILE *_TheFile = fopen(TheFile, "rb");
   if (_TheFile==NULL)
   {
      strcpy(tmpstr,"|31| Error reading file: ");
      strcat(tmpstr,TheFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   FILE *_RAWFile = fopen(RAWFile, "wb+");
   if (_RAWFile==NULL)
   {
      strcpy(tmpstr,"|31| Can't create file: ");
      strcat(tmpstr,RAWFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   ProgressDialog = new TProgressDialog("Convert BASIC file");
   deskTop->insert(ProgressDialog);

   fseek(_TheFile, 0, 2);
   long fz = ftell(_TheFile);
   fseek(_TheFile, 0, 0);

   for(unsigned short int w = 1; w <= 0x1E00; w++)
   {
       Write_bit_1(_RAWFile);
   }
   for(w = 1; w <= 10; w++)
   {
       Write_byte(_RAWFile, 0xD3);
   }
   WriteFileName(_RAWFile, MSXFile);
   for(w = 0; w < 0xFFFF; w++)
   {
       fwrite(&nul0, sizeof(nul0), 1, _RAWFile);
   }
   for(w = 1; w <= 0x780; w++)
   {
       Write_bit_1(_RAWFile);
   }

   unsigned char B;
   unsigned long fpos = 0;
   while(fpos < fz)
   {
      fread(&B, sizeof(B), 1, _TheFile);
      Write_byte(_RAWFile, B);
      fpos = ftell(_TheFile);
      ProgressDialog->update((100 * fpos) / fz);
   }

   Write_byte(_RAWFile, 0x00);
   Write_byte(_RAWFile, 0x00);
   Write_byte(_RAWFile, 0x00);

   fclose(_TheFile);
   fclose(_RAWFile);

   destroy(ProgressDialog);
   InfoWindow->writeLine("|31| Successfully!!!");
}

void TKorvetApp::Bin2Msx()
{
   TProgressDialog *ProgressDialog;
   char titul[MAXPATH];
   char RAWFile[MAXPATH];

   char MSXFile[MAXPATH];
   char tmpstr[255];

   short int nul0 = 0;

   strcpy(titul, TheFile);
   strcpy(tmpstr, "|31| Reading file ");
   strcat(tmpstr, TheFile);
   InfoWindow->writeLine(tmpstr);
   ChangeFileExt(RAWFile, TheFile, ".RAW");
   strcpy(tmpstr, "|31| Convert source to MSX tape binary file ");
   strcat(tmpstr, RAWFile);
   InfoWindow->writeLine(tmpstr);
   InfoWindow->writeLine("|31| Open this file in your Wave Editor or Player with following params:");
   InfoWindow->writeLine("|31| Sample rate: 44100 Hz");
   InfoWindow->writeLine("|31| Sample type: 16 Bit PCM");
   InfoWindow->writeLine("|31| Signed");
   InfoWindow->writeLine("|31| Byte order: Little-endian");
   InfoWindow->writeLine("|31| Chanels: Mono");
   strcpy(tmpstr, "|31| Set MSX filename to [");
   GetMSXFileName(MSXFile, RAWFile);
   strcat(tmpstr, MSXFile);
   strcat(tmpstr, "]");
   InfoWindow->writeLine(tmpstr);
   InfoWindow->writeLine("|31| In PK8020/PK8010 KORVET you can read this file:");
   strcpy(tmpstr, "|31| LOAD \"CAS:");
   strcat(tmpstr, MSXFile);
   strcat(tmpstr, "\",B[,R]");
   InfoWindow->writeLine(tmpstr);
   sprintf(tmpstr,"|31| Set load addres to %Xh",LoadAddr);
   InfoWindow->writeLine(tmpstr);
   sprintf(tmpstr,"|31| Set start addres to %Xh",StartAddr);
   InfoWindow->writeLine(tmpstr);

   FILE *_TheFile = fopen(TheFile, "rb");
   if (_TheFile==NULL)
   {
      strcpy(tmpstr,"|31| Error reading file: ");
      strcat(tmpstr,TheFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   FILE *_RAWFile = fopen(RAWFile, "wb+");
   if (_RAWFile==NULL)
   {
      strcpy(tmpstr,"|31| Can't create file: ");
      strcat(tmpstr,RAWFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   ProgressDialog = new TProgressDialog("Convert binary file");
   deskTop->insert(ProgressDialog);

   fseek(_TheFile, 0, 2);
   long int fz = ftell(_TheFile);
   fseek(_TheFile, 0, 0);

   for(unsigned short int w = 1; w <= 0x1E00; w++)
   {
       Write_bit_1(_RAWFile);
   }
   for(w = 1; w <= 10; w++)
   {
      Write_byte(_RAWFile, 0xD0);
   }
   WriteFileName(_RAWFile, MSXFile);
   for(w = 0; w < 0xFFFF; w++)
   {
       fwrite(&nul0, sizeof(nul0), 1, _RAWFile);
   }
   for(w = 1; w <= 0x780; w++)
   {
       Write_bit_1(_RAWFile);
   }

   EndAddr   = fz + LoadAddr - 1;

   Write_byte(_RAWFile, LoadAddr & 0xFF);
   Write_byte(_RAWFile, LoadAddr >> 8);
   Write_byte(_RAWFile, EndAddr & 0xFF);
   Write_byte(_RAWFile, EndAddr >> 8);
   Write_byte(_RAWFile, StartAddr & 0xFF);
   Write_byte(_RAWFile, StartAddr >> 8);

   unsigned char B;
   unsigned long fpos = 0;
   while(fpos < fz)
   {
      fread(&B, sizeof(B), 1, _TheFile);
      Write_byte(_RAWFile, B);
      fpos = ftell(_TheFile);
      ProgressDialog->update((100 * fpos) / fz);
   }
   fclose(_TheFile);
   fclose(_RAWFile);
   destroy(ProgressDialog);
   InfoWindow->writeLine("|31| Successfully!!!");
}

void TKorvetApp::Msr2Wav()
{
   TProgressDialog *ProgressDialog;
   char titul[MAXPATH];
   char WAVFile[MAXPATH];

   char tmpstr[255];

   strcpy(titul, TheFile);
   strcpy(tmpstr, "|31| Reading file ");
   strcat(tmpstr, TheFile);
   InfoWindow->writeLine(tmpstr);
   ChangeFileExt(WAVFile, TheFile, ".WAV");
   strcpy(tmpstr, "|31| Convert MSR to WAV file");
   strcat(tmpstr, WAVFile);
   InfoWindow->writeLine(tmpstr);


   FILE *_TheFile = fopen(TheFile, "rb");
   if (_TheFile==NULL)
   {
      strcpy(tmpstr,"|31| Error reading file: ");
      strcat(tmpstr,TheFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   FILE *_WAVFile = fopen(WAVFile, "wb+");
   if (_WAVFile==NULL)
   {
      strcpy(tmpstr,"|31| Can't create file: ");
      strcat(tmpstr,WAVFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   ProgressDialog = new TProgressDialog("Convert MSR to WAV");
   deskTop->insert(ProgressDialog);

   fseek(_TheFile, 0, 2);
   long int fz = ftell(_TheFile);
   fseek(_TheFile, 0, 0);

   TWaveHeader wh;
   wh.idRiff   = 0x46464952;
   wh.RiffLen  = fz+sizeof(TWaveHeader);
   wh.idWave   = 0x45564157;
   wh.idFmt    = 0x20746D66;
   wh.InfoLen  = 0x10;
   wh.WaveType = 1;
   wh.Ch       = 1;
   wh.Freq     = 44100;
   wh.BytesPerSec = 2*44100;
   wh.align    = 2;
   wh.Bits     = 16;

   TDataHeader dh;
   dh.idData  = 0x61746164;
   dh.DataLen = fz;

   fwrite(&wh, sizeof(wh), 1, _WAVFile);
   fwrite(&dh, sizeof(dh), 1, _WAVFile);

   short int C[255];
   unsigned long fpos = 0;
   while(fpos < fz)
   {
      fpos = fz - fpos;
      if(fpos > sizeof(C))
      {
         fpos = sizeof(C);
      }
      fread(&C, fpos , 1, _TheFile);
      fwrite(&C, fpos, 1, _WAVFile);
      fpos = ftell(_TheFile);
      ProgressDialog->update((100 * fpos) / fz);
   }
   fclose(_TheFile);
   fclose(_WAVFile);
   destroy(ProgressDialog);
   InfoWindow->writeLine("|31| Successfully!!!");
}

void TKorvetApp::MSXFileNew()
{
   strcpy(TheFile, "*.*");
   TFileDialog *FileDialog = new TFileDialog(TheFile, "Open file", "~F~ile name", fdOpenButton, 1);
   if(execDialog(FileDialog, TheFile) != cmCancel)
   {
      FILE *f = fopen(TheFile, "rb");
      if(!f)
      {
         messageBox("\3Selected File not found", mfError + mfOKButton);
         return;
      }
      fclose(f);
      char titul[MAXPATH];
      strcpy(titul, TheFile);
      InfoWindow = new TInfoWindow(titul);
      deskTop->insert(InfoWindow);
      char S[MAXEXT];
      GetFileExt(S, TheFile);
      if(strstr(".TXT.BAS.PAS.DOC",S) != NULL)
      {
         Txt2Msx();
      }
      if(strstr(".CAS", S) != NULL)
      {
         Cas2Msx();
      }
      if(strstr(".BIN", S) != NULL)
      {
         Bin2Msx();
      }
      if(strstr(".COM", S) != NULL)
      {
         messageBox("\3Sorry, in this moment present only bin2msr conversion", mfOKButton);
         Bin2Msx();
      }
      if(strstr(".MSR", S) != NULL)
      {
         Msr2Wav();
      }
   }
}

void TKorvetApp::RAWRecode()
{
   char RAWFile[MAXPATH];

   char tmpstr[255];
   TProgressDialog *ProgressDialog;

   strcpy(tmpstr, "|31| Reading file ");
   strcat(tmpstr, TheFile);
   InfoWindow->writeLine(tmpstr);

   ChangeFileExt(RAWFile, TheFile, ".MSR");
   strcpy(tmpstr, "|31| Recode source MSX tape file to ");
   strcat(tmpstr, RAWFile);
   InfoWindow->writeLine(tmpstr);

   FILE *_TheFile = fopen(TheFile, "rb");
   if (_TheFile==NULL)
   {
      strcpy(tmpstr,"|31| Error reading file: ");
      strcat(tmpstr,TheFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }
   FILE *_RAWFile = fopen(RAWFile, "wb+");
   if (_RAWFile==NULL)
   {
      strcpy(tmpstr,"|31| Can't create file: ");
      strcat(tmpstr,RAWFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   ProgressDialog = new TProgressDialog("Recode RAW file");
   deskTop->insert(ProgressDialog);

   fseek(_TheFile, 0, 2);
   long int fz = ftell(_TheFile);
   fseek(_TheFile, 0, 0);

   short int C[256];
   unsigned long fpos = 0;
   unsigned short int j;
   while(fpos < fz)
   {
      fpos = fz - fpos;
      if(fpos > sizeof(C))
      {
         fpos = sizeof(C);
      }
      fread(&C, fpos /*sizeof(C)*/, 1, _TheFile);
      for(j = 0; (j < /*sizeof(C)*/ fpos / 2); j++)
      {
         if(C[j] > 0)
         {
            C[j] = 5000;
         }
         if(C[j] < 0)
         {
            C[j] = -5000;
         }
      }
      fwrite(&C, /*sizeof(C)*/ fpos, 1, _RAWFile);
      fpos = ftell(_TheFile);
      ProgressDialog->update((100 * fpos) / fz);
   }
   fclose(_TheFile);
   fclose(_RAWFile);
   destroy(ProgressDialog);
   InfoWindow->writeLine("|31| You can rename *.MSR file to *.RAW file for writing to cassette");
}

void TKorvetApp::TX1Decode()
{
   char TX1File[MAXPATH];
   char tmpstr[255];
   TProgressDialog *ProgressDialog;

   strcpy(tmpstr, "|31| Process phase 1 file ");
   strcat(tmpstr, TheFile);
   strcat(tmpstr, " decode");
   InfoWindow->writeLine(tmpstr);

   ChangeFileExt(TX1File, TheFile, ".TX1");
   strcpy(tmpstr, "|31| Set output file to ");
   strcat(tmpstr, TX1File);
   InfoWindow->writeLine(tmpstr);
   strcpy(tmpstr, "|31| Please wait...It''s a very long operation");
   InfoWindow->writeLine(tmpstr);

   FILE *_TheFile = fopen(TheFile, "rb");
   if (_TheFile==NULL)
   {
      strcpy(tmpstr,"|31| Error reading file: ");
      strcat(tmpstr,TheFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }
   FILE *_TX1File = fopen(TX1File, "wb+");
   if (_TX1File==NULL)
   {
      strcpy(tmpstr,"|31| Can't create file: ");
      strcat(tmpstr,TX1File);
      InfoWindow->writeLine(tmpstr);
      return;
   }
   ProgressDialog = new TProgressDialog("Phase 1 decode");
   deskTop->insert(ProgressDialog);

   fseek(_TheFile, 0, 2);
   long int fz = ftell(_TheFile);
   fseek(_TheFile, 0, 0);

   short int C;
   unsigned long fpos = 0;
   unsigned char counter = 0;
   while(fpos < fz)
   {
      fread(&C, sizeof(C), 1, _TheFile);
      if(C < 0)
      {
         counter++;
      }
      else
      {
         if((counter > 5) && (counter < 14))
         {
            counter = '1';
            fwrite(&counter, sizeof(counter), 1, _TX1File);
         }
         if((counter > 14) && (counter < 30))
         {
            counter = '0';
            fwrite(&counter, sizeof(counter), 1, _TX1File);
         }
         counter = 0;
      }
      fpos = ftell(_TheFile);
      ProgressDialog->update((100 * fpos) / fz);
   }
   fclose(_TheFile);
   fclose(_TX1File);
   destroy(ProgressDialog);
   InfoWindow->writeLine("|31| TX1 file create successfully");
}

void TKorvetApp::TX2Decode()
{
   char TX1File[MAXPATH];
   char TX2File[MAXPATH];
   char tmpstr[255];
   TProgressDialog *ProgressDialog;

   ChangeFileExt(TX1File, TheFile, ".TX1");

   strcpy(tmpstr, "|31| Process phase 2 file ");
   strcat(tmpstr, TheFile);
   strcat(tmpstr, " decode");
   InfoWindow->writeLine(tmpstr);

   ChangeFileExt(TX2File, TheFile, ".TX2");
   strcpy(tmpstr, "|31| Set output file to ");
   strcat(tmpstr, TX2File);
   InfoWindow->writeLine(tmpstr);
   strcpy(tmpstr, "|31| Please wait...It''s a very long operation");
   InfoWindow->writeLine(tmpstr);

   FILE *_TX1File = fopen(TX1File, "rb");
   if (_TX1File==NULL)
   {
      strcpy(tmpstr,"|31| Error reading file: ");
      strcat(tmpstr,TX1File);
      InfoWindow->writeLine(tmpstr);
      return;
   }
   FILE *_TX2File = fopen(TX2File, "wb+");
   if (_TX2File==NULL)
   {
      strcpy(tmpstr,"|31| Can't create file: ");
      strcat(tmpstr,TX2File);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   ProgressDialog = new TProgressDialog("Phase 2 decode");
   deskTop->insert(ProgressDialog);

   fseek(_TX1File, 0, 2);
   long int fz = ftell(_TX1File);
   fseek(_TX1File, 0, 0);

   unsigned char C;
   unsigned long fpos = 0;
   unsigned char is1 = 0;

   while(fpos < fz)
   {
      fread(&C, sizeof(C), 1, _TX1File);
      fpos = ftell(_TX1File);
      ProgressDialog->update((100 * fpos) / fz);
      if(C == '0')
      {
         fwrite(&C, sizeof(C), 1, _TX2File);
         is1 = 0;
         continue;
      }
      if(is1 == 1)
      {
         is1--;
      }
      else
      {
         is1++;
         fwrite(&C, sizeof(C), 1, _TX2File);
      }
   }
   fclose(_TX1File);
   fclose(_TX2File);
   destroy(ProgressDialog);
   InfoWindow->writeLine("|31| TX2 file create successfully");
}

void TKorvetApp::CASDecode()
{
   char TX2File[MAXPATH];
   char CASFile[MAXPATH];
   char tmpstr[255];
   TProgressDialog *ProgressDialog;

   ChangeFileExt(TX2File, TheFile, ".TX2");

   strcpy(tmpstr, "|31| Open file ");
   strcat(tmpstr, TX2File);
   strcat(tmpstr, " for reading");
   InfoWindow->writeLine(tmpstr);

   FILE *_TX2File = fopen(TX2File, "rb");
   if (_TX2File==NULL)
   {
      strcpy(tmpstr,"|31| Error reading file: ");
      strcat(tmpstr,TX2File);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   ProgressDialog = new TProgressDialog("Process create CAS file");
   deskTop->insert(ProgressDialog);

   fseek(_TX2File, 0, 2);
   long int fz = ftell(_TX2File);
   fseek(_TX2File, 0, 0);
   unsigned long fpos = 0;
   unsigned char C;
   do
   {
      fread(&C, sizeof(C), 1, _TX2File);
      fpos = ftell(_TX2File);
      ProgressDialog->update((100 * fpos) / fz);
      if(C == '0')
      {
         break;
      }
   } while(1 == 1);
   fseek(_TX2File, -1, 1);

   unsigned char F_Type[10];

   for(C = 0; C <= 9; C++)
   {
      F_Type[C] = ReadByte(_TX2File);
   }
   for(C = 0; C <= 9; C++)
   {
      char jx[10];
      strcpy(tmpstr, "|31| Found FileType ");
      sprintf(jx,"%i",C);
      strcat(tmpstr, jx);
      strcat(tmpstr, " ");
      sprintf(jx,"%Xh",F_Type[C]);
      strcat(tmpstr, jx);
      InfoWindow->writeLine(tmpstr);
   }

   switch(F_Type[9])
//   switch(F_Type[0])
   {
      case 0xd3:
      {
         for(C = 0; C <= 5; C++)
         {
            CASFile[C] = ReadByte(_TX2File);
         }
         CASFile[6] = 0;
         strcpy(tmpstr, "|31| Found MSX Basic file ");
         strcat(tmpstr, CASFile);
         InfoWindow->writeLine(tmpstr);
         InfoWindow->writeLine("|31| You can rename it");
                        ChangeFileExt(CASFile, TheFile, ".CAS");
         strcpy(tmpstr, "|31| Processing CAS file create ");
         strcat(tmpstr, CASFile);
         InfoWindow->writeLine(tmpstr);

         do
         {
            fread(&C, sizeof(C), 1, _TX2File);
            fpos = ftell(_TX2File);
            ProgressDialog->update((100 * fpos) / fz);
            if(C == '0')
            {
               break;
            }
         } while(1 == 1);
         fseek(_TX2File, -1, 1);

         FILE *_CASFile = fopen(CASFile, "wb+");

         while(fpos < fz)
         {
            C = ReadByte(_TX2File);
            fwrite(&C, sizeof(C), 1, _CASFile);
            fpos = ftell(_TX2File);
            ProgressDialog->update((100 * fpos) / fz);
         }
         fclose(_CASFile);
         InfoWindow->writeLine("|31| CAS file create successfully");
         break;
      }

      case 0xd0:
      {
         for(C = 0; C <= 5; C++)
         {
            CASFile[C] = ReadByte(_TX2File);
         }
         CASFile[6] = 0;
         strcpy(tmpstr, "|31| Found MSX binary file ");
         strcat(tmpstr, CASFile);
         InfoWindow->writeLine(tmpstr);
         InfoWindow->writeLine("|31| You can rename it");
         ChangeFileExt(CASFile, TheFile, ".BIN");
         strcpy(tmpstr, "|31| Processing BIN file create ");
         strcat(tmpstr, CASFile);
         InfoWindow->writeLine(tmpstr);

         do
         {
            fread(&C, sizeof(C), 1, _TX2File);
            fpos = ftell(_TX2File);
            ProgressDialog->update((100 * fpos) / fz);
            if(C == '0')
            {
               break;
            }
         } while(1 == 1);
         fseek(_TX2File, -1, 1);

         unsigned short int LoadAddr  = (unsigned short int)(ReadByte(_TX2File)) + (unsigned short int)(ReadByte(_TX2File) << 8);
         unsigned short int EndAddr   = (unsigned short int)(ReadByte(_TX2File)) + (unsigned short int)(ReadByte(_TX2File) << 8);
         unsigned short int StartAddr = (unsigned short int)(ReadByte(_TX2File)) + (unsigned short int)(ReadByte(_TX2File) << 8);

         char s[20];

         strcpy(tmpstr, "|31| Load addres is ");
         sprintf(s,"%Xh",LoadAddr);
         strcat(tmpstr, s);
         InfoWindow->writeLine(tmpstr);

         strcpy(tmpstr, "|31| End addres is ");
         sprintf(s,"%Xh",EndAddr);
         strcat(tmpstr, s);
         InfoWindow->writeLine(tmpstr);

         strcpy(tmpstr, "|31| Start addres is ");
         sprintf(s,"%Xh",StartAddr);
         strcat(tmpstr, s);
         InfoWindow->writeLine(tmpstr);
         FILE *_CASFile = fopen(CASFile, "wb+");
         while(fpos < fz)
         {
            C = ReadByte(_TX2File);
            fwrite(&C, sizeof(C), 1, _CASFile);
            fpos = ftell(_TX2File);
            ProgressDialog->update((100 * fpos) / fz);
         }
         fclose(_CASFile);
         InfoWindow->writeLine("|31| BIN file create successfully");
         break;
      }

      case 0xEA:
      {
         for(C = 0; C <= 5; C++)
         {
            CASFile[C] = ReadByte(_TX2File);
         }
         CASFile[6] = 0;
         strcpy(tmpstr, "|31| Found MSX Text file ");
         strcat(tmpstr, CASFile);
         InfoWindow->writeLine(tmpstr);
         InfoWindow->writeLine("|31| You can rename it");
         ChangeFileExt(CASFile, TheFile, ".TXT");
         strcpy(tmpstr, "|31| Processing TXT file create ");
         strcat(tmpstr, CASFile);
         InfoWindow->writeLine(tmpstr);

         do
         {
            fread(&C, sizeof(C), 1, _TX2File);
            fpos = ftell(_TX2File);
            ProgressDialog->update((100 * fpos) / fz);
            if(C == '0')
            {
               break;
            }
         } while(1 == 1);
         fseek(_TX2File, -1, 1);

         FILE *_CASFile = fopen(CASFile, "wb+");
         unsigned char j2 = 0;
         while(fpos < fz)
         {
            if(j2 == 0)
            {
               do
               {
                  fread(&C, sizeof(C), 1, _TX2File);
                  fpos = ftell(_TX2File);
                  ProgressDialog->update((100 * fpos) / fz);
                  if(C == '0')
                  {
                     break;
                  }
               } while(1 == 1);
               fseek(_TX2File, -1, 1);
            }
            C = ReadByte(_TX2File);
            fpos = ftell(_TX2File);
            ProgressDialog->update((100 * fpos) / fz);
            if(C == 0x1A)
            {
               break;
            }
            fwrite(&C, sizeof(C), 1, _CASFile);
            j2++;
         }
         fclose(_CASFile);
         InfoWindow->writeLine("|31| TXT file create successfully");
         break;
      }
   }
   fclose(_TX2File);
   destroy(ProgressDialog);
}

void TKorvetApp::BASDecode()
{
   char CASFile[MAXPATH];
   char BASFile[MAXPATH];

   char tmpstr[255];
   TProgressDialog *ProgressDialog;

   ChangeFileExt(CASFile, TheFile, ".CAS");
   ChangeFileExt(BASFile, TheFile, ".BAS");

   strcpy(tmpstr, "|31| Open file ");
   strcat(tmpstr, CASFile);
   strcat(tmpstr, " for reading");
   InfoWindow->writeLine(tmpstr);
   strcpy(tmpstr, "|31| Processing BAS file create ");
   strcat(tmpstr, BASFile);
   InfoWindow->writeLine(tmpstr);

   FILE *_CASFile = fopen(CASFile, "rb");
   if (_CASFile==NULL)
   {
      strcpy(tmpstr,"|31| Error reading file: ");
      strcat(tmpstr,CASFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   FILE *_BASFile = fopen(BASFile, "wt+");
   if (_BASFile==NULL)
   {
      strcpy(tmpstr,"|31| Can't create file: ");
      strcat(tmpstr,BASFile);
      InfoWindow->writeLine(tmpstr);
      return;
   }

   ProgressDialog = new TProgressDialog("Process create BAS file");
   deskTop->insert(ProgressDialog);

   fseek(_CASFile, 0, 2);
   long int fz = ftell(_CASFile);
   fseek(_CASFile, 0, 0);

   unsigned long fpos = 0;
   unsigned short int nor = 0;
   unsigned short int LineNumber = 0;
   char S[255];

   fread(&nor, 1, 1, _CASFile);
   if (nor!=0xFF) fseek(_CASFile, -1, 1);

   while(fpos < fz)
   {
      fread(&nor, sizeof(nor), 1, _CASFile);
      if (nor==0x0000) break;
      fread(&LineNumber, sizeof(LineNumber), 1, _CASFile);
      fprintf(_BASFile,"%u ",LineNumber);
      ReadLine(_CASFile,S);
      fprintf(_BASFile,"%s\n",S);
   }
   fclose(_CASFile);
   fclose(_BASFile);
   destroy(ProgressDialog);
   InfoWindow->writeLine("|31| BAS file create successfully");
}

struct ToptData
{
   char _LoadAddr[6];
   char _StartAddr[6];

};

void TKorvetApp::setOptions()
{
   TOptDialog *optDialog;
   ToptData *optData;

   char _tmp[128];

   optData = new ToptData;

   sprintf(_tmp,"%Xh",LoadAddr);
   strcpy( optData->_LoadAddr, _tmp);
   sprintf(_tmp,"%Xh",StartAddr);
   strcpy( optData->_StartAddr,_tmp);

   optDialog = new TOptDialog();
   optDialog->setData( optData );
   if (deskTop->execView( optDialog ) != cmCancel)
   {
      optDialog->getData( optData );
      LoadAddr  = strtoul(optData->_LoadAddr , NULL, 16);
      StartAddr = strtoul(optData->_StartAddr, NULL, 16);
   }

   delete optData;

}

void main()
{
   TKorvetApp KorvetApp;
   KorvetApp.run();
}
